/**
 * Licensed Materials - Property of IBM
 * 
 * OCO Source Materials
 * 
 * license number t.b.d
 * 
 * (C) Copyright IBM Corp. 2009 All Rights Reserved
 * 
 * The source code for this program is not published or other- wise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 * 
 **/
package com.ibm.director.adt.ovf.test;

import static org.junit.Assert.*;

import java.io.File;
import java.io.IOException;

import org.dmtf.schemas.ovf.envelope.EnvelopeType;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.ibm.director.adt.ovf.exceptions.OVFException;
import com.ibm.director.adt.ovf.utils.OVFUtils;

/**
 * 
 * @author jantony
 * @since TODO: Add version
 * 
 */
public class OVFBasicTest extends BaseOVFTest {

	private String ovfFilePath = null;
	private File mFile = null;

	// to execute from command line - todo: later
	public static void main(String[] args) {

	}

	/**
	 * Called once for every JUnit Run
	 */
	@BeforeClass
	public static void setupOnce() {

		System.out.println("setupOnce() called");

	}

	/**
	 * Called once for every JUnit Test
	 */
	@Before
	public void setup() {

		System.out.println("setup() called");

		ovfFilePath = null;
	}
	
	/**
	 * Called once after each JUnit Test
	 */
	@After
	public void tearDown() {
		
		System.out.println("teardown() called");

		mFile = null;

	}
	
	/**
	 * Called once after junit run
	 */
	@AfterClass
	public static void tearDownAfterTests() {
		
		System.out.println("teardownAfterTests() called");
	}
	

	/**
	 * Test: OVF Utility should load the valid OVF file. 
	 * Result: Should load the file. No exceptions thrown
	 */

	@Test
	public void testLoadFromFile() {

		ovfFilePath = "src/test/resources/OVF/chuck.ovf"; // "ovfTestFile.ovf";

		String test_loadFile = "loadFromFile(\"" + ovfFilePath + "\")";
		System.out.println("Try: " + test_loadFile);
		
		System.out.println("Current working dir: " + System.getProperty("user.dir"));
	
		EnvelopeType env = null;

		try {

			assertTrue("ovfTestFile path is not readable.", 
					BaseOVFTestUtil.getFileFromPath(ovfFilePath).canRead());

			env = OVFUtils.loadFromFile(ovfFilePath);

			assertNotNull(("Failed to load OVF File: " + ovfFilePath), env);

		} catch (OVFException oe) {
			fail("Exception loading OVF File: " + ovfFilePath);
			oe.printStackTrace();
		} catch (IOException e) {
			fail("Exception loading OVF File: " + ovfFilePath);
			e.printStackTrace();
		}

	}

	/**
	 * Test: OVF Utility should throw and exception if given an invalid OVF file
	 * to load. Result: This should throw an exception
	 */

	@Test
	public void testLoadFromInvalidOVFFile() {

		String ovfFilePath = "src/test/resources/OVF/badchuck.ovf";

		String test_loadFile = "loadFromFile(" + ovfFilePath + ")";
		System.out.println("Try: " + test_loadFile);

		EnvelopeType env = null;

		try {

			assertTrue("ovfTestFile path is not readable.", 
					BaseOVFTestUtil.getFileFromPath(ovfFilePath).canRead());

			env = OVFUtils.loadFromFile(ovfFilePath);

			fail("testLoadFromInvalidFile: should throw exception.");

		} catch (OVFException oe) {
			assertNull(("Failed to load OVF File: " + ovfFilePath), env);
			oe.printStackTrace();
		} catch (IOException e) {
			fail("testLoadFromInvalidFile: IOException");
			e.printStackTrace();
		}

	}


	/**
	 * Test: OVF Utility should throw and exception if given a file that is not readable
	 * to load. Result: This should throw an exception
	 */

	@Test
	public void testLoadFromNonExistantFile() {
		
		
		String ovfFilePath = "ovfTestNonExistantFile.ovf";

		String test_loadFile = "loadFromFile(" + ovfFilePath + ")";
		System.out.println("Try: " + test_loadFile);

		EnvelopeType env = null;

		try {

			// no assertion on readabilty of the file - let it pass  through
			/*assertTrue("ovfTestFile path is not readable.", 
					getFileFromPath(ovfFilePath).canRead());*/

				env = OVFUtils.loadFromFile(ovfFilePath);
			
			fail("testLoadFromInvalidFile: should throw exception.");

		} catch (OVFException oe) {
			assertNull(("Failed to load OVF File: " + ovfFilePath), env);

			// oe.printStackTrace();
		} catch (IOException e) {
			
			fail("testLoadFromInvalidFile: Caught IOException, should throw OVFException.");
			
			// e.printStackTrace();
		}

	}
	
	
	
	/**
	 * Test: Generation of default OVF envelope. Result: Generate a default OVF
	 * envelope. No exceptions thrown.
	 */

	@Test
	public void testGenerateDefaultEnvelopeType() {

		String ovfFilePath = "src/test/resources/OVF/chuck.ovf";
		
		String test = "generateDefaultEnvelopeType(" + ovfFilePath + ")";
		System.out.println("Try: " + test);

		EnvelopeType env = null;

		try {

			assertTrue("ovfTestFile path is not readable.", 
					BaseOVFTestUtil.getFileFromPath(ovfFilePath).canRead());
			
			try {
				env = OVFUtils.generateDefaultOVFEnvelope(ovfFilePath);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			assertNotNull(
					("Failed to generate default envelope from OVF File: " + ovfFilePath),
					env);

		} catch (OVFException oe) {
			fail("Exception loading OVF File: " + ovfFilePath);
			oe.printStackTrace();
		}
	}



}
