package com.ibm.director.adt.ovf.test;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.dmtf.schemas.ovf.envelope.DocumentRoot;
import org.dmtf.schemas.ovf.envelope.EnvelopeType;
import org.dmtf.schemas.ovf.envelope.FileType;
import org.dmtf.schemas.ovf.envelope.OperatingSystemSectionType;
import org.dmtf.schemas.ovf.envelope.ReferencesType;
import org.dmtf.schemas.ovf.envelope.VirtualHardwareSectionType;
import org.eclipse.emf.common.util.Diagnostic;
import org.eclipse.emf.common.util.EMap;

import com.ibm.director.adt.ovf.constants.OVFConstants;
import com.ibm.director.adt.ovf.data.OVFProductProperty;
import com.ibm.director.adt.ovf.data.OVFProductSection;
import com.ibm.director.adt.ovf.data.OVFVirtualHWItem;
import com.ibm.director.adt.ovf.exceptions.OVFException;
import com.ibm.director.adt.ovf.utils.OVFUtils;

import java.net.URI;
import java.net.URISyntaxException;

public class OVFConsoleTest {

  /**
   * @param args
   */
  public static void main(String[] args) throws IOException {
	// Load OVF file given on the command line.
	if (args.length == 0) {
	  System.out.println("Please provide an XML descriptor.");
	  System.exit(0);
	}
	
	String path = args[0].toString(); 
	  
	//
	// Test: Open XML descriptor and create envelope.
	//
	EnvelopeType env = null;	
	env = testLoadFromFile(path);

	testCheckResourceType(env, path);
	//	EnvelopeType env = testLoadFromString(path);
		  	  
	//
	// Test: Load XML descriptor as string. 
	//

	
	//
	// Test: Default OVF generation.
	//
	//	testGenerateDefaultOVFEnvelope();
	// some comment  

	//
	// Test: setReferenceHandle 
	//
	//testSetReferenceHandle(path, env);


	//
	// Test: getVirtualHardwareSectionItemsValues(EnvelopeType, 
	//                                            List<OVFVirtualHWItem> items)
	//
	testGetVirtualHardwareSectionItemsValues(env);

	
	//
	// Test: Extended attributes.
	//
	//testAddFileReference(env);
	

	//
	// Test: XML Environment file.
	//
	testBuildEnv(env);
	
	//
	// Test: getProductPropertiesForCustomizationList 
	//
	testGetProductPropertiesForCustomizationList(env);

	//
	// Test: addVHWItemExtElems
	//
	// testAddVHWItemExtElems(env);

	//
	// Test: Extended attributes.
	//
	testGetExtVHWAttributes(env);

	//
	// Test: get vim:description
	//
	// testGetVimDescription(env);
	
	//
	// Test: Add a ProductSection
	//
	
	testCheckProdSection(env);
	
	//
	// Test: set vim:description
	//
	// testSetVimDescription(env);

	//
	// Test: change vim:description
	//
	testChangeVimDescription(env);
	
	//
	// Test: remove vim:description
	//
	//testRmVimDescription(env);
	// testGetVimDescription(env);

	//
	// Test: Add OS section
	//
//	OperatingSystemSectionType os = testAddOsSection(env);

	//
	// Test: vim OS elements
	//
//	testVimOsAttrs(env, os);

	OperatingSystemSectionType os = testGetOS(env);
	testGetVimOsAttrs(env, os);
	
	//
	// Test: GetFilteredItemHashMapList
	//
	
//	testGetFilteredItemHashMapList(env);

	//
	// Test: Get product name
	//
	testGetProdName(env);
	
	//
	// Test: File output in current directory.
	//
//	testSaveToFile(env, path);

	//
	// Test: saveToBytes
	//
//	testSaveToBytes(env, path);
  }


  public static void testGetProdName(EnvelopeType env) {
	  System.out.println("Try: testGetProdName");
	  String p = OVFUtils.getProductName(env);
	  System.out.println ("Product: '" + p + "'");
  }
    
  public static void printNsMap(EnvelopeType env) {
	  // Print namespace map

	  DocumentRoot docroot = (DocumentRoot)env.eContainer();
	  EMap<String, String> xmlMap = docroot.getXMLNSPrefixMap();
	  Map<String, String> map = xmlMap.map();

	  Iterator<Map.Entry<String, String>> it = map.entrySet().iterator();
	  System.out.println("ns map:");
	  while (it.hasNext()) {
		  Map.Entry <String, String> pairs = (Map.Entry <String,String>)it.next();
		  System.out.println("   " + pairs.getKey() + " = " + pairs.getValue());
	  }
  }
  
  public static void testGetFilteredItemHashMapList(EnvelopeType env) {
	  System.out.println("Try: testGetFilteredItemHashMapList");

	   List <HashMap <String,String>> items;

	   HashMap <String,String> networkItemsFilter = new HashMap <String,String>();
	   // networkItemsFiltenr.put("resourcetype", "10");
	   items = OVFUtils.getFilteredItemHashMapList (env, null, networkItemsFilter);
	   // List <HashMap <String,String>> items = getFilteredItemHashMapList(env, "LINUX206" , null);
	   for (HashMap <String,String> item: items) {
	      System.out.println("Item: " + item.get("instanceid"));
	      Iterator<Map.Entry<String, String>> it = item.entrySet().iterator();
	      while (it.hasNext()) {
	         Map.Entry <String, String> pairs = (Map.Entry <String,String>)it.next();
	         System.out.println("   " + pairs.getKey() + " = " + pairs.getValue());
	      }
	   }
  }
  
  public static OperatingSystemSectionType testGetOS(EnvelopeType env) {
	  System.out.println("Try: testGetOS");
	  OperatingSystemSectionType os = OVFUtils.getOperatingSystem(env);
	  System.out.println("OS info: " + os.getInfo().getValue());
	  return os;
  }
  
  public static void testCheckResourceType(EnvelopeType env, String fileName) {
	  System.out.println("Try: testCheckResourceType");
	  try {
		OVFUtils.checkResourceTypes(env, fileName);
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
  }
  
  public static void testGetProductPropertiesForCustomization(EnvelopeType env) {
	  System.out.println("Try: testGetGetProductPropertiesForCustomization");
	  
	  HashMap<String, OVFProductSection> p = null;
	try {
		p = OVFUtils.getProductPropertiesForCustomization(env);
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	  
	  for (String cls : p.keySet()) {
		  OVFProductSection ps = p.get(cls);
		  System.out.println(cls);
		  HashMap<String,OVFProductProperty> keyMap = ps.getVimKeyMap();
		  for (String vimKey : keyMap.keySet()) {
			  System.out.println("\t" + vimKey+"="+keyMap.get(vimKey).getValue());
			  keyMap.get(vimKey).setValue("test_ovf_val");
		  }
		  
		  HashMap<String, List<OVFProductProperty>> catMap = ps.getProperties();
		  for (String cat : catMap.keySet()) {
			  System.out.println("\t" +"category: '"+cat+"'");
			  for (OVFProductProperty prop : catMap.get(cat)) {
				  System.out.println(prop.getVIMKey());
			  }
		  }		  
	  }
	  
	  testBuildEnv(env);
  }
  
  public static void testGetProductPropertiesForCustomizationList(EnvelopeType env) {
	  System.out.println("Try: testGetGetProductPropertiesForCustomizationListList");
	  
	  List<OVFProductSection> lst = null;
	try {
		lst = OVFUtils.getProductPropertiesForCustomizationList(env);
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	  
	  for (OVFProductSection p : lst) {
		  System.out.println(p.getOvfClass());
		  HashMap<String,OVFProductProperty> keyMap = p.getVimKeyMap();
		  for (String vimKey : keyMap.keySet()) {
			  System.out.println("\t" + vimKey+"="+keyMap.get(vimKey).getValue());
			  keyMap.get(vimKey).setValue("test_ovf_val");
		  }
		  
		  HashMap<String, List<OVFProductProperty>> catMap = p.getProperties();
		  for (String cat : catMap.keySet()) {
			  System.out.println("\t" +"category: '"+cat+"'");
			  for (OVFProductProperty prop : catMap.get(cat)) {
				  System.out.println(prop.getVIMKey());
			  }
		  }		  
	  }
	  
	  testBuildEnv(env);
  }  
  
  public static void testGetVimOsAttrs(EnvelopeType env, OperatingSystemSectionType os) {
	  System.out.println("Try: testVimOsAttrs");

	  List<String> l = OVFUtils.getDiscoveryIpAddress(os);
	if ((l != null) && (l.size() > 0)) {
		System.out.println("discoveryIpAddress key: " + l.get(0) + " class: " + l.get(1) + " instance: " + l.get(2));
	}
	l = OVFUtils.getNetworkBootIpAddress(os);
	if ((l != null) && (l.size() > 0)) {
		System.out.println("NetworkBootIpAddress key: " + l.get(0) + " class: " + l.get(1) + " instance: " + l.get(2));
	}
  }
  
  public static void testVimOsAttrs(EnvelopeType env, OperatingSystemSectionType os) {
	  System.out.println("Try: testVimOsAttrs");

	  OVFUtils.setDiscoveryIpAddress(os, "tst_key1", "tst_class", "tst_instance");
	  OVFUtils.setNetworkBootIpAddress(os, "tst_key2", "tst_class", "tst_instance");
  } 

  public static OperatingSystemSectionType testAddOsSection(EnvelopeType env) {
	  System.out.println("Try: testAddOsSection");
	  OperatingSystemSectionType os = 
		  OVFUtils.addOperatingSystemSection(env, "Tst_id", "Tst_version", "Tst_info", "Tst_description");
	  return os;
  }

  public static void testCheckProdSection(EnvelopeType env) {
	  System.out.println("Try: testCheckProdSection");
	  try {
		OVFUtils.checkProductSection(env, "file");
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
  }
  
  public static void testRmVimDescription(EnvelopeType env) {
	  System.out.println("Try: testRmVimDescription");
	  OVFUtils.setVimDescription(env, null);
  }
  
  public static void testChangeVimDescription(EnvelopeType env) {
	  System.out.println("Try: testChangeVimDescription");
	  OVFUtils.setVimDescription(env, "new description");
  }
    
  public static void testSetVimDescription(EnvelopeType env) {
	  System.out.println("Try: testSetVimDescription");
	  OVFUtils.setVimDescription(env, "OVF_console_test");
  }
    
  public static void testGetVimDescription(EnvelopeType env) {
	  System.out.println("Try: testGetVimDescription");
	  String desc = OVFUtils.getVimDescription(env);
	  String msgid = OVFUtils.getVimDescriptionMsgId(env);
	  System.out.println("vim:description="+desc+"msgid="+msgid);
  }
  
  public static void testAddVHWItemExtElems(EnvelopeType env) {
	  System.out.println("Try: testAddVHWItemExtElems");
	  
	  List<OVFVirtualHWItem> items = new ArrayList<OVFVirtualHWItem>();
	  OVFUtils.getVirtualHardwareSectionItemsValues(env, items);
	  
	  try {
		  for (OVFVirtualHWItem item : items) {
			  OVFUtils.setPhypRasdShareMode(item.getItem(), env, "uncap");
			  
			  System.out.println("New share mode: " +
								 OVFUtils.getPhypRasdShareMode(item.getItem()));
			  
			  OVFUtils.setPhypRasdQuantity(item.getItem(), env, "42");
			  System.out.println("New quantity: " +
								 OVFUtils.getPhypRasdQuantity(item.getItem()));

			  OVFUtils.setPhypRasdVirtualLimit(item.getItem(), env, "43");
			  System.out.println("New Virtual Limit: " +
								 OVFUtils.getPhypRasdVirtualLimit(item.getItem()));
								 
			  OVFUtils.setPhypRasdVirtualReservation(item.getItem(), env, "44");
			  System.out.println("New virtual reservation: " +
								 OVFUtils.getPhypRasdVirtualReservation(item.getItem()));
		  }
		  
	  } catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
  }
  
  public static void testBuildEnv(EnvelopeType env) {
	  System.out.println("Try: testBuildEnv");
	  String xmlEnv = null;
	  try {
		xmlEnv = OVFUtils.buildEnvironmentXMLStr(env);
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	System.out.println(xmlEnv);
  }
  
  public static void testSaveToFile(EnvelopeType env, String fname) {
	  System.out.println("Try: testSaveToFile");
	  String outFile = fname + ".out";
	  
	  try {
		OVFUtils.saveToFileByFullPath(env, outFile);
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
  }

  public static void testSaveToBytes(EnvelopeType env, String fname) {
	  System.out.println("Try: testSaveToBytes");
	  String outFile = fname + ".bytes";
	  FileOutputStream outFileStream = null;
	  
	  try {
		outFileStream = new FileOutputStream( outFile );
	} catch (FileNotFoundException e1) {
		// TODO Auto-generated catch block
		e1.printStackTrace();
	}
	  
	  byte[] ovfBytes = null;
	  
	  try {
		ovfBytes = OVFUtils.saveToBytes(env);
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	
	try {
		outFileStream.write(ovfBytes);
	} catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	
  }

  public static void testAddFileReference(EnvelopeType env) {
	  System.out.println("Try: testAddFileReference");
	  System.out.println("Before file addition");
	  
	  HashMap<String,FileType> files = new HashMap<String, FileType>();
	  
	  OVFUtils.getFileReferences(env, files);
	  for (String id : files.keySet()) {
		  System.out.println(files.get(id).getHref());
	  }
	  
	  try {
		OVFUtils.addFileReference(env, "OVF_NEW_HREF", new BigInteger("16000"));
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	
	  System.out.println("after file addition");
	  
	  files.clear();
	  
	  OVFUtils.getFileReferences(env, files);
	  for (String id : files.keySet()) {
		  System.out.println(files.get(id).getHref());
	  }
  }
  
  public static void testGetExtVHWAttributes(EnvelopeType env) {
	  
	  System.out.println("Try: testGetAllVHWAttributes");
	  List<String> a = new ArrayList<String>();
	  
	  a.add("http://www.ibm.com/xmlns/ovf/extension/vim/2/phyp/3/rasd");
	  a.add(OVFConstants.OVF_NS_VIMZVMRASD_URI);
	  
	  HashMap<String, HashMap<String, String>> hm = OVFUtils.getExtVHWItemAttributes(env, a);
	  System.out.println("Found "+hm.size()+" items with extension attributes.");
	  
	  for (String resourceType : hm.keySet()) {
		  HashMap<String, String> attrMap = hm.get(resourceType);
		  for (String attrName : attrMap.keySet()) {
			  System.out.println(resourceType+": "+attrName+"="+attrMap.get(attrName));
		  }			  
	  }	  
  }
  
  
  public static void testGetVirtualHardwareSectionItemsValues(EnvelopeType env) {

	System.out.println("Try: testGetVirtualHardwareSectionItemsValues");
	
	ArrayList<OVFVirtualHWItem> items = new ArrayList<OVFVirtualHWItem>();

	VirtualHardwareSectionType vhw =
	  OVFUtils.getVirtualHardwareSectionItemsValues(env, items);

	if (vhw == null) {
	  System.out.println("No VirtualHardwareSection entities in OVF.");
	  return;
	} // if (vhw == null) 

	for (OVFVirtualHWItem item : items) {
	  System.out.println(item.getLabel());
	} // for (OVFVirtualHWItem item : items)


	// Individual RASDTypes

  } // public static void testGetVirtualHardwareSectionItemsValues(EnvelopeType env) 
  
  public static EnvelopeType testLoadFromString(String path) {
	  
	  String test_loadString = "loadFromString("+path+")";
	  System.out.println("Try: "+test_loadString);

	  EnvelopeType env = null;
	  InputStream stream = null;
	try {
		stream = new FileInputStream(path);
	} catch (FileNotFoundException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	
	StringWriter sw = new StringWriter();
	BufferedReader br = new BufferedReader(new InputStreamReader(stream));
	String line = null;
	try {
		while ((line = br.readLine()) != null) {
			sw.write(line);
			sw.write("\n");
		}
		sw.write("\n");
	} catch (IOException e1) {
		// TODO Auto-generated catch block
		e1.printStackTrace();
	}
		
	try {
		env = OVFUtils.loadFromString(sw.getBuffer().toString());
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	return env;
	  
  }
  
  public static EnvelopeType testLoadFromFile(String path) {
	
	String test_loadFile = "loadFromFile("+path+")";
	System.out.println("Try: "+test_loadFile);
	EnvelopeType env = null;
	Diagnostic d = null;
	
	try {

		// file name is not url
		if(!path.startsWith("file:")&&
				!path.startsWith("http:")&&
				!path.startsWith("https:")
		){
			System.out.println("try loadFromFile: "+ path );
			try {
			  try {
				env = OVFUtils.loadFromFile(path);
				OVFUtils.semanticValidate(env, path, d);
			} catch (IOException e) {
				// TODO Auto-generated catch block
				System.out.println("Fail: " + test_loadFile + ": " + e.getMessage() + d.getMessage());
				System.exit(1);
			}
			} catch (OVFException oe) {
			  System.out.println("Fail: " + test_loadFile + ": " + oe.getMessage());
			  System.exit(1);
			} 
			
		} else
		{   // file is url reference
			URI hrefURI = null;
			try {
				hrefURI = new URI(path);
			} catch (URISyntaxException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				System.exit(1);
			}

			String scheme = hrefURI.getScheme();

			System.out.println("Get inputstream for "
						+ path + 
						" uri is " + hrefURI + " via scheme: "+scheme);

			if (scheme == null) {
				System.out.println("loadFromFile (url ref)");
				env = OVFUtils.loadFromFile(path);

			} else if (scheme.equalsIgnoreCase("file")) {
				// treat the path as absolute
				System.out.println("loadFromFile (file ref)");
				env = OVFUtils.loadFromFile(hrefURI.getPath());
				
			} else if (scheme.equalsIgnoreCase("http")||scheme.equalsIgnoreCase("https")) {
				// load package over http
				System.out.println("loadFromURL (http or https)");
				env = OVFUtils.loadFromURL(path);

			} // scheme has to be "http" or "https" 
			  // has to be since this is TAR file and if import gets this far 
			  // this is ova package file and was already found and opened to get the
			  // ovf descriptor envelop. So know file name has to follow at least
			  // one of the three valid naming schemes or was not found
		}
	} catch (OVFException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	} catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}
	
	System.out.println("Success: " + test_loadFile);
	return env;
  } // public static void testLoadFromFile(String path) 

  
  public static void testSetReferenceHandle(String ovf_descriptor,
											EnvelopeType env) {

	// Print out existing references in the file.
	// get to file reference section for envelope
		  
		  
	ReferencesType ref = env.getReferences();
	int numOfRef = ref.getFile().size();
	//	EList<FileType> fileCopy = new EList<FileType>(ref.getFile());
	
	String currentFileId;
	String currentHref;
	
	String test_setReferenceHandle =
	  "setReferenceHandle("+ovf_descriptor+",fileId, handle, pkgId";
			

	// Change every id to the handle
	System.out.println("\nPrinting " + numOfRef + " reference[s]");
			
	// find reference for fileId in envelope references
	for (int i = 0; i < numOfRef; i++) {

	  currentFileId = ref.getFile().get(i).getId();
	  currentHref = ref.getFile().get(i).getHref();
	  
	  System.out.println("\n\t"+currentFileId+":"+currentHref);

	  // Change this file's href to the "handle"
	  try {
		OVFUtils.setReferenceHandle(env, currentFileId, "handle", "pkgId");
	  }
	  catch(OVFException oe) {
		System.out.println("Fail: " + test_setReferenceHandle + ": " + oe.getMessage());
		System.exit(1);		  
	  } // catch 

	  // Did the change take?
	  currentFileId = ref.getFile().get(i).getId();
	  currentHref = ref.getFile().get(i).getHref();
	  System.out.println("\t"+currentFileId+":"+currentHref);
	  
	  
	} // for (int i = 0; i < numOfRef; i++) 
  }

  public static void testGenerateDefaultOVFEnvelope()  {
	String me = "testGenerateDefaultOVFEnvelope()";

	//	String currentdir = System.getProperty("user.dir");
	//	System.out.println(currentdir);
	System.out.println("Trying: " + me);
	EnvelopeType env = null;
	try {
	  String ovfPath = "default_customizable_ovf.xml";
	  env = OVFUtils.generateDefaultOVFEnvelope(ovfPath);
	} // try
   	catch(OVFException oe) {
	  System.out.println("Fail: " + me + ": " + oe.getMessage());
	} // catch
 catch (IOException e) {
		// TODO Auto-generated catch block
		e.printStackTrace();
	}

	System.out.println("OS: " + OVFUtils.getOperatingSystemValue(env, "defaultOS"));
	
  } // public static void testGenerateDefaultOVFEnvelope()

} // public class OVFTest 

