package com.ibm.director.adt.ovf.test;

import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringWriter;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.dmtf.schemas.ovf.envelope.EnvelopeType;
import org.dmtf.schemas.ovf.envelope.FileType;
import org.dmtf.schemas.ovf.envelope.ReferencesType;
import org.dmtf.schemas.ovf.envelope.VirtualHardwareSectionType;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import com.ibm.director.adt.ovf.data.OVFVirtualHWItem;
import com.ibm.director.adt.ovf.exceptions.OVFException;
import com.ibm.director.adt.ovf.utils.OVFUtils;

/**
 * Licensed Materials - Property of IBM
 * 
 * OCO Source Materials
 * 
 * license number t.b.d
 * 
 * (C) Copyright IBM Corp. 2009 All Rights Reserved
 * 
 * The source code for this program is not published or other- wise divested of
 * its trade secrets, irrespective of what has been deposited with the U.S.
 * Copyright Office.
 * 
 **/

/**
 *
 * @author jantony
 * @since TODO: Add version
 *
 */
public class OVFUtilTest {
	
	private static EnvelopeType ovfEnv = null;

	/**
	 * @throws java.lang.Exception
	 */
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		String ovfFilePath = "chuck.ovf";
		
		ovfEnv =  loadOVFEnvFromFile(ovfFilePath); // testLoadFromString(ovfFilePath);
		
		assertNotNull("OVF envelop is null", ovfEnv);
		
	}

	/**
	 * @throws java.lang.Exception
	 */
	@AfterClass
	public static void tearDownAfterClass() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@Before
	public void setUp() throws Exception {
	}

	/**
	 * @throws java.lang.Exception
	 */
	@After
	public void tearDown() throws Exception {
	}
	
	/**
	 * Test: Build Environment from XML String.
	 */
	@Test
	public void testBuildEnvironmentFromXMLString() {
		System.out.println("Try: testBuildEnvironmentFromXMLString");
		
		String xmlEnv = null;
		
		try {
			assertNotNull("OVF envelope not initialized.", ovfEnv);
			
			xmlEnv = OVFUtils.buildEnvironmentXMLStr(ovfEnv);
			
			assertNotNull("xmlEnv from buildEnvironmentXMLStr is null.", xmlEnv);
			
		} catch (OVFException e) {
			
			fail("Unable to build environment from XML String.");
			e.printStackTrace();
		}
		System.out.println(xmlEnv);
	}
	
	/**
	 * Test: [Negative] Build Environment from XML String.
	 */
	@Test
	public void testBuildEnvironmentFromXMLString_nullEnv() {
		System.out.println("Try: testBuildEnvironmentFromXMLString");
		
		String xmlEnv = null;
		
		try {
			assertNotNull("OVF envelope not initialized.", ovfEnv);
			
			xmlEnv = OVFUtils.buildEnvironmentXMLStr(null); // pass null ovfEnv - should fail gracefully.
			
			fail("OVFUtils.buildEnvironmentXMLStr should throw an exception");
			
		} catch (OVFException e) {
			
			assertTrue("Caught expected OVFException.", Boolean.TRUE);
			e.printStackTrace();
		} catch (Exception e) {

			fail("Unexpected exception thrown for OVFUtils.buildEnvironmentXMLStr");
			e.printStackTrace();
		}
		System.out.println(xmlEnv);
	}	
	
	
	/**
	 * Test: Save to File by Full Path
	 * 
	 */
	@Test
	public void testSaveToFileByFullPath() {
		System.out.println("Try: testSaveToFileByFullPath");

		String outFilePath = "testSaveToFileByFullPath.out";

		try {
			assertNotNull("OVF envelope not initialized.", ovfEnv);

			OVFUtils.saveToFileByFullPath(ovfEnv, outFilePath);

			assertTrue("Successfully saved to file by full path", Boolean.TRUE);

		} catch (OVFException e) {

			fail("Unexpected OVFException when saving to file by full path.");
			e.printStackTrace();
		}
	}
	
	/**
	 * Test: Save to Bytes
	 * 
	 */
	@Test
	public void testSaveToBytes() {
		System.out.println("Try: testSaveToBytes");

		String outFile = "testSaveToBytes.bytes";
		FileOutputStream outFileStream = null;
		byte[] ovfBytes = null;

		try {
			outFileStream = new FileOutputStream(outFile);
			assertNotNull("Unable to create new FileOutputStream", outFileStream);
		
			ovfBytes = OVFUtils.saveToBytes(ovfEnv);
			assertNotNull("Unable to save OVF to bytes", ovfBytes);
	
			outFileStream.write(ovfBytes);
			
			assertTrue("Successfully wrote bytes to output file", Boolean.TRUE);
			
		} catch (OVFException e) {
			e.printStackTrace();
			fail("Unexpected OVFException when saving bytes to output file.");
		} catch (IOException e) {
			e.printStackTrace();
			fail("Unexpected IOException when saving bytes to output file.");
		}
	}
	
	/**
	 * Test: Add File Reference
	 * 
	 */
	@Test
	  public  void testAddFileReference() {
		System.out.println("Try: testAddFileReference");
		System.out.println("Before file addition");

		HashMap<String, FileType> files = new HashMap<String, FileType>();
		
		assertNotNull("Hashmap to hold file information is null.", files);

		OVFUtils.getFileReferences(ovfEnv, files);
		
		System.out.println("# of files: " + files.size());
		for (String id : files.keySet()) {
			System.out.println(files.get(id).getHref());
		}

		try {
			OVFUtils.addFileReference(ovfEnv, 
					"OVF_NEW_HREF", 
					new BigInteger("16000"));
			
			assertTrue(Boolean.TRUE);
		} catch (OVFException e) {
			
			e.printStackTrace();
			fail("Unexpected OVFException when adding a file reference to an OVF.");
		}
		
		assertTrue(Boolean.TRUE);

		System.out.println("after file addition");

		
		// todo: not sure what we are looking for here. 
		files.clear();

		OVFUtils.getFileReferences(ovfEnv, files);

		System.out.println("# of files: " + files.size());
		for (String id : files.keySet()) {
			System.out.println(files.get(id).getHref());
		}
	}

	/**
	 * Test: Get Ext VHWAttributes todo: might want to enhance this test or have
	 * another test for the following: - test for specific size of the attribute
	 * set. - test for specific name/value pair of attributes. This test only
	 * verifies that the attributes could be retrieved from the ovf.
	 */
	@Test
	public void testGetExtVHWAttributes() {

		System.out.println("Try: testGetAllVHWAttributes");
		List<String> a = new ArrayList<String>();

		assertNotNull(a);

		a.add("http://www.ibm.com/xmlns/ovf/extension/vim/2/phyp/3/rasd");

		HashMap<String, HashMap<String, String>> hm = OVFUtils
				.getExtVHWItemAttributes(ovfEnv, a);

		System.out.println("Found " + hm.size()
				+ " items with extension attributes.");

		for (String resourceType : hm.keySet()) {
			HashMap<String, String> attrMap = hm.get(resourceType);

			assertNotNull(attrMap);
			for (String attrName : attrMap.keySet()) {
				System.out.println(resourceType + ": " + attrName + "="
						+ attrMap.get(attrName));
			}
		}
	}

	/**
	 * Test: Get Virtual Hardware Section Items uValues
	 * 
	 */
	@Test
	public void testGetVirtualHardwareSectionItemsValues() {

		System.out.println("Try: testGetVirtualHardwareSectionItemsValues");

		ArrayList<OVFVirtualHWItem> items = new ArrayList<OVFVirtualHWItem>();

		VirtualHardwareSectionType vhw = OVFUtils
				.getVirtualHardwareSectionItemsValues(ovfEnv, items);

		assertNotNull("No VirtualHardwareSection entities in OVF.", vhw);

		for (OVFVirtualHWItem item : items) {
			System.out.println(item.getLabel());
		}// for (OVFVirtualHWItem item : items)
	}
	

	/**
	 * Test: Load ovf from String
	 * 
	 */
	@Test
	public void testLoadFromString() {

		String ovfFilePath = "chuck.ovf";

		String test_loadString = "loadFromString(" + ovfFilePath + ")";
		System.out.println("Try: " + test_loadString);

		EnvelopeType env = null;
		InputStream stream = null;
		try {
			stream = new FileInputStream(ovfFilePath);

			assertNotNull(stream);

			StringWriter sw = new StringWriter();
			BufferedReader br = new BufferedReader(
					new InputStreamReader(stream));

			assertNotNull(sw);
			assertNotNull(br);

			String line = null;
			while ((line = br.readLine()) != null) {
				sw.write(line);
				sw.write("\n");
			}
			sw.write("\n");

			env = OVFUtils.loadFromString(sw.getBuffer().toString());

			assertNotNull(env);

		} catch (FileNotFoundException e) {

			e.printStackTrace();
			fail("Failed to create FileInputStream from: " + ovfFilePath);
		} catch (IOException e1) {

			e1.printStackTrace();
			fail("Failed to write to the FileInputStream: " + ovfFilePath);
		} catch (OVFException e) {

			e.printStackTrace();
			fail("Unexpected OVF exception.");
		}

	}
	

	 /**
	 * Test: Load ovf from File
	 * 
	 */
	@Test
	public void testLoadFromFile() {

		String ovfFilePath = "chuck.ovf";

		String test_loadFile = "loadFromFile(" + ovfFilePath + ")";
		System.out.println("Try: " + test_loadFile);

		EnvelopeType env = null;

		try {
			env = OVFUtils.loadFromFile(ovfFilePath);

			assertNotNull(env);

//			OVFUtils.semanticValidate(env);

		} catch (IOException e) {

			e.printStackTrace();
			fail("IOException when loading OVF from file.");
		} catch (OVFException oe) {

			oe.printStackTrace();
			fail("Failed to validate the OVF.");
		}
	} 
	
	 /**
	 * Test: Set Reference Handle
	 * 
	 */
	@Test
	public void testSetReferenceHandle() { 

		// Print out existing references in the file.
		// get to file reference section for envelope

		assertNotNull(ovfEnv);

		ReferencesType ref = ovfEnv.getReferences();
		assertNotNull(ref);

		int numOfRef = ref.getFile().size();

		String currentFileId = null;
		String currentHref = null;


		// Change every id to the handle
		System.out.println("\nPrinting " + numOfRef + " reference[s]");

		// find reference for fileId in envelope references
		for (int i = 0; i < numOfRef; i++) {

			currentFileId = ref.getFile().get(i).getId();
			currentHref = ref.getFile().get(i).getHref();

			assertNotNull(currentFileId);
			assertNotNull(currentHref);

			System.out.println("\n\t" + currentFileId + ":" + currentHref);

			// Change this file's href to the "handle"
			try {

				OVFUtils.setReferenceHandle(ovfEnv, currentFileId, "handle",
						"pkgId");
				assertTrue(Boolean.TRUE);
			} catch (OVFException oe) {

				oe.printStackTrace();
				fail("Unexpected OVFException.");
			}

			// Did the change take?
			currentFileId = ref.getFile().get(i).getId();
			currentHref = ref.getFile().get(i).getHref();
			assertTrue(currentHref.equals("handle"));
			System.out.println("\t" + currentFileId + ":" + currentHref);

		}
	}

	
	
	
	/**
	 * loads an OVF Envelope from a file. 
	 * used for tests. called once for this test suite
	 * @return envelope loaded from the file.
	 */
	private static EnvelopeType loadOVFEnvFromFile(String ovfFilePath) {

		System.out.println("Current working dir: " + System.getProperty("user.dir"));
	
		EnvelopeType env = null;

		try {

			assertTrue("ovfTestFile path is not readable.", 
					BaseOVFTestUtil.getFileFromPath(ovfFilePath).canRead());

			env = OVFUtils.loadFromFile(ovfFilePath);

			assertNotNull(("Failed to load OVF File: " + ovfFilePath), env);

		} catch (OVFException oe) {
			fail("Exception loading OVF File: " + ovfFilePath);
			oe.printStackTrace();
		} catch (IOException e) {
			fail("Exception loading OVF File: " + ovfFilePath);
			e.printStackTrace();
		}
		
		return env;

	}

}
